from .Widget import Widget

from .WT_PUSH_BUTTON import WT_PUSH_BUTTON
from .WT_LABEL import WT_LABEL
from .WT_TOGGLE import WT_TOGGLE
from .WT_SLIDER import WT_SLIDER
from .WT_PROGRESS_BAR import WT_PROGRESS_BAR
from .WT_GROUP import WT_GROUP
from .WT_ICON_LIST import WT_ICON_LIST
from .WT_SCROLLABLE_CAMERA import WT_SCROLLABLE_CAMERA
from .WT_SCROLLABLE_SET import WT_SCROLLABLE_SET
from .WT_CANVAS import WT_CANVAS
from .WT_FINGER_CATCHER import WT_FINGER_CATCHER

WidgetTypes = {
    'WT_PUSH_BUTTON': WT_PUSH_BUTTON,
    'WT_LABEL': WT_LABEL,
    'WT_TOGGLE': WT_TOGGLE,
    'WT_SLIDER': WT_SLIDER,
    'WT_PROGRESS_BAR': WT_PROGRESS_BAR,
    'WT_GROUP': WT_GROUP,
    'WT_ICON_LIST': WT_ICON_LIST,
    'WT_SCROLLABLE_CAMERA': WT_SCROLLABLE_CAMERA,
    'WT_SCROLLABLE_SET': WT_SCROLLABLE_SET,
    'WT_CANVAS': WT_CANVAS,
    'WT_FINGER_CATCHER': WT_FINGER_CATCHER,
}
